"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.deleteObject = exports.objectRegistrationInfo = void 0;
const AdtException_1 = require("../AdtException");
const utilities_1 = require("../utilities");
async function objectRegistrationInfo(h, objectUrl) {
    (0, AdtException_1.ValidateObjectUrl)(objectUrl);
    const response = await h.request("/sap/bc/adt/sscr/registration/objects", {
        qs: { uri: objectUrl }
    });
    const raw = (0, utilities_1.fullParse)(response.body)["reg:objectRegistrationResponse"];
    return {
        developer: (0, utilities_1.xmlNodeAttr)(raw["reg:developer"]),
        object: (0, utilities_1.xmlNodeAttr)(raw["reg:object"]),
        ...(0, utilities_1.xmlNodeAttr)(raw)
    };
}
exports.objectRegistrationInfo = objectRegistrationInfo;
async function deleteObject(h, objectUrl, lockHandle, transport) {
    (0, AdtException_1.ValidateObjectUrl)(objectUrl);
    (0, AdtException_1.ValidateStateful)(h);
    const qs = { lockHandle };
    if (transport)
        qs.corrNr = transport;
    const method = "DELETE";
    // no return value, will throw on failure
    await h.request(objectUrl, { method, qs });
}
exports.deleteObject = deleteObject;
